//==============================================
//      File dbmore.pas
//
//      Components library for Delphi and BCB++.
//      Creating TBDEDataset clones.
//
//      Copyright 1998-2001 Polaris Software
//      http://members.xoom.com/PolarisSoft
//      mailto: PolarisLib@mail.ru
//==============================================
unit DBMore;

{$I POLARIS.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, BDE, rDBConst;

type
  TrCloneTable = class(TTable)
  private
    FSourceTable: TTable;
    NewIndexFieldNames, NewIndexName: string;
    procedure SetSourceTable(Value: TTable);
  protected
    function CreateHandle: HDBICur; override;
    procedure OpenCursor(InfoQuery: Boolean); override;
    procedure DoAfterOpen; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
  published
    property SourceTable: TTable read FSourceTable write SetSourceTable;
  end;

  TrCloneQuery = class(TQuery)
  private
    FSourceDataSet: TDBDataSet;
    procedure SetSourceDataSet(Value: TDBDataSet);
  protected
    function CreateHandle: HDBICur; override;
    procedure OpenCursor(InfoQuery: Boolean); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
  published
    property SourceDataSet: TDBDataSet read FSourceDataSet write SetSourceDataSet;
  end;

implementation

{ TrCloneTable }

procedure TrCloneTable.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FSourceTable = AComponent) then FSourceTable := nil;
end;

function TrCloneTable.CreateHandle: HDBICur;
begin
  if FSourceTable <> nil then
    Check(DbiCloneCursor(FSourceTable.Handle, ReadOnly, False, Result))
  else Result := inherited CreateHandle;
end;

procedure TrCloneTable.OpenCursor(InfoQuery: Boolean);
begin
  NewIndexFieldNames := '';
  NewIndexName := '';
  if FSourceTable <> nil then begin
    if not FSourceTable.Active then FSourceTable.Open;
    DatabaseName := FSourceTable.DatabaseName;
    ReadOnly   := FSourceTable.ReadOnly;
    TableName  := FSourceTable.TableName;
    TableType  := FSourceTable.TableType;
    UpdateMode := FSourceTable.UpdateMode;
    if (IndexFieldNames <> '') and
       (IndexFieldNames <> FSourceTable.IndexFieldNames) then begin
      NewIndexFieldNames := IndexFieldNames;
      IndexFieldNames := '';
    end;
    if (IndexName <> '') and (IndexName <> FSourceTable.IndexName) then begin
      NewIndexName := IndexName;
      IndexName := '';
    end;
  end;
  inherited OpenCursor(InfoQuery);
end;

procedure TrCloneTable.DoAfterOpen;
begin
  if NewIndexName <> '' then IndexName := NewIndexName
  else
    if NewIndexFieldNames <> '' then IndexFieldNames := NewIndexFieldNames;
  inherited;
end;

procedure TrCloneTable.SetSourceTable(Value: TTable);
begin
  if Value <> FSourceTable then begin
    CheckInActive;
    FSourceTable := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

{ TrCloneQuery }

procedure TrCloneQuery.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FSourceDataSet = AComponent) then FSourceDataSet := nil;
end;

function TrCloneQuery.CreateHandle: HDBICur;
begin
  if FSourceDataSet <> nil then
    Check(DbiCloneCursor(FSourceDataSet.Handle, True, False, Result))
  else
    Result := inherited CreateHandle;
end;

procedure TrCloneQuery.OpenCursor(InfoQuery: Boolean);
begin                               
  if FSourceDataSet <> nil then begin
    if not FSourceDataSet.Active then FSourceDataSet.Open;
    DatabaseName := FSourceDataSet.DatabaseName;
    if FSourceDataSet is TQuery then
      SQL.Text := TQuery(FSourceDataSet).Text
    else if FSourceDataSet is TTable then
      SQL.Text := 'SELECT * FROM '+TTable(FSourceDataSet).TableName;
  end;
  inherited OpenCursor(InfoQuery);
end;

procedure TrCloneQuery.SetSourceDataSet(Value: TDBDataSet);
begin
  if Value <> FSourceDataSet then begin
    CheckInActive;
    FSourceDataSet := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

end.
