//==============================================
//      File dbspinedit.pas
//
//      Components library for Delphi and BCB++.
//      DBAware   TSpinEdit.
//      DBAware realization of TSpinEdit.
//
//      Copyright 1998-2001 Polaris Software
//      http://members.xoom.com/PolarisSoft
//      mailto: PolarisLib@mail.ru
//==============================================
unit DBSpinEdit;

{$I POLARIS.INC}

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DBCtrls, DB, RXSpin;

type
  TCustomDBSpinEdit = class(TRxCustomSpinEdit)
  private
    FDataLink: TFieldDataLink;
    FValueTypeAuto,
    FFocused: Boolean;
    FAlignment: TAlignment;
    FCanvas: TControlCanvas;

    procedure ActiveChange(Sender: TObject);
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);
    procedure SetValueTypeAuto(Value: Boolean);

    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetReadOnly(Value: Boolean);

    procedure SetFocused(Value: Boolean);
    procedure ResetMaxLength;

    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;

    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    function GetTextMargins: TPoint;

    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure WMUndo(var Message: TMessage); message WM_UNDO;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
//    function GetFormatedText: String;
    function RealValueType: TValueType;
    procedure CheckValueType;
    procedure SetValueType(NewType: TValueType); override;
    procedure SetValue(NewValue: Extended); override;
    function GetValue: Extended; override;
    function GetReadOnly: Boolean;
    procedure Change; override;

    procedure UpClick(Sender: TObject); override;
    procedure DownClick(Sender: TObject); override;

    function EditCanModify: Boolean; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    function IsValidChar(Key: Char): Boolean; override;

    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Reset; override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
{$IFDEF POLARIS_D4}
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    function UseRightToLeftAlignment: Boolean; override;
{$ENDIF}
    property Field: TField read GetField;
//  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ValueTypeAuto: Boolean read FValueTypeAuto write SetValueTypeAuto default True;
    property ButtonKind default bkClassic;
  end;

  TDBSpinEdit = class(TCustomDBSpinEdit)
  published
//Polaris
    property Align;

    property CheckOnExit;
    property CheckMinValue;
    property CheckMaxValue;

    property DataField;
    property DataSource;
    property ReadOnly;
    property ValueTypeAuto;
    property ArrowKeys;
    property ButtonKind;
//    property Decimal;
    property EditorEnabled;
    property Increment;
    property MaxValue;
    property MinValue;
    property ValueType default {$IFDEF POLARIS_BCB}vtInt{$ELSE}vtInteger{$ENDIF};
//    property Value;
    property OnBottomClick;
    property OnTopClick;

    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
{$IFDEF POLARIS_D4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
{$ENDIF}
    property ImeMode;
    property ImeName;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
{$IFDEF RX_D5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF RX_D4}
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

implementation

{ TCustomDBSpinEdit }

procedure TCustomDBSpinEdit.CMEnter(var Message: TCMEnter);
begin
  SetFocused(True);
  inherited;
  if SysLocale.FarEast and FDataLink.CanModify then
    inherited ReadOnly := False;
end;

procedure TCustomDBSpinEdit.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SelectAll;
    SetFocus;
    raise;
  end;
  SetFocused(False);
  CheckCursor;
  DoExit;
end;

procedure TCustomDBSpinEdit.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

constructor TCustomDBSpinEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  inherited ReadOnly := True;
  FValueTypeAuto := True;
//  ValueType := vtFloat;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
  FDataLink.OnActiveChange := ActiveChange;
  FButtonKind := bkClassic;
  RecreateButton;
end;

procedure TCustomDBSpinEdit.ActiveChange(Sender: TObject);
begin
  ResetMaxLength;
end;

procedure TCustomDBSpinEdit.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil then
  begin
    if FAlignment <> FDataLink.Field.Alignment then
    begin
//      Value := 0.0;
      SetValue(0.0);
      EditText := '';  {forces update}
      FAlignment := FDataLink.Field.Alignment;
    end;
    EditMask := FDataLink.Field.EditMask;
    if not (csDesigning in ComponentState) then
    begin
      if (FDataLink.Field.DataType in [ftString{$IFDEF POLARIS_D4}, ftWideString{$ENDIF}]) and (MaxLength = 0) then
        MaxLength := FDataLink.Field.Size;
    end;
    if FFocused and FDataLink.CanModify then begin
//      Text := FDataLink.Field.Text;
//      Value := StrToFloat(Text);
      if FDataLink.Field.IsNull then begin
        SetValue(0.0);
        Text := '';
      end else SetValue(FDataLink.Field.Value);
    end else
    begin
      if FDataLink.Field.IsNull
      then begin
        SetValue(0.0);
        EditText := '';
      end
      else begin
        SetValue(FDataLink.Field.Value);
        if not FValueTypeAuto and (ValueType=vtHex)
        then EditText := IntToHex(Round(FDataLink.Field.AsFloat),0)
        else EditText := FDataLink.Field.DisplayText;
      end

//      if FDataLink.Editing and FDataLink.FModified then
//        Modified := True;
    end;
  end else
  begin
    FAlignment := taLeftJustify;
    SetValue(0.0);
    EditMask := '';
    if csDesigning in ComponentState then begin
      EditText := Name;
//      {Self.}Value := 0.0;
    end else EditText := '';
  end;
end;

destructor TCustomDBSpinEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FCanvas.Free;
  inherited Destroy;
end;

procedure TCustomDBSpinEdit.EditingChange(Sender: TObject);
begin
  inherited ReadOnly := not FDataLink.Editing;
end;

{$IFDEF POLARIS_D4}
function TCustomDBSpinEdit.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TCustomDBSpinEdit.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

function TCustomDBSpinEdit.UseRightToLeftAlignment: Boolean;
begin
  Result := DBUseRightToLeftAlignment(Self, Field);
end;

{$ENDIF}

function TCustomDBSpinEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

function TCustomDBSpinEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TCustomDBSpinEdit.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TCustomDBSpinEdit.SetDataField(const Value: string);
begin
  if not (csDesigning in ComponentState) then ResetMaxLength;
  FDataLink.FieldName := Value;
  CheckValueType;
end;

procedure TCustomDBSpinEdit.CheckValueType;
begin
  if FValueTypeAuto
  then ValueType := RealValueType;
end;

function TCustomDBSpinEdit.RealValueType: TValueType;
begin
  Result := ValueType;
  if ((csDesigning in ComponentState) or FValueTypeAuto) and
  Assigned(FDataLink) and Assigned(FDataLink.Field)
  then if (FDataLink.Field is TFloatField)
    then Result := vtFloat
    else Result := {$IFDEF POLARIS_BCB}vtInt{$ELSE}vtInteger{$ENDIF};
end;

procedure TCustomDBSpinEdit.SetDataSource(Value: TDataSource);
begin
{$IFDEF POLARIS_D4}
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
{$ENDIF}
    FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

procedure TCustomDBSpinEdit.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then
  begin
    FFocused := Value;
    if (FAlignment <> taLeftJustify) and not IsMasked then Invalidate;
    FDataLink.Reset;
  end;
end;

procedure TCustomDBSpinEdit.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

procedure TCustomDBSpinEdit.UpdateData(Sender: TObject);
begin
  ValidateEdit;
  if Trim(Text) = EmptyStr then FDataLink.Field.Clear
  else FDataLink.Field.Value := Value;
end;

procedure TCustomDBSpinEdit.WMCut(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TCustomDBSpinEdit.WMPaste(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TCustomDBSpinEdit.WMUndo(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

function TCustomDBSpinEdit.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TCustomDBSpinEdit.Change;
begin
  FDataLink.Modified;
  inherited Change;
end;

function TCustomDBSpinEdit.EditCanModify: Boolean;
begin
  Result := FDataLink.Edit;
end;

procedure TCustomDBSpinEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
    FDataLink.Edit;
end;

function TCustomDBSpinEdit.IsValidChar(Key: Char): Boolean;
begin
  Result := False;
  if RealValueType = vtHex then inherited IsValidChar(Key)
  else begin
    if (RealValueType = vtFloat) and (Key='.') then Key := DecimalSeparator;
    Result := inherited IsValidChar(Key);//FDataLink.Field.IsValidChar(Key);
  end
end;

procedure TCustomDBSpinEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Key in [#32..#255]) and (FDataLink.Field <> nil) and
    not IsValidChar(Key) then
  begin
    Beep;
    Key := #0;
  end;
  case Key of
    ^H, ^V, ^X, #32..#255:
      FDataLink.Edit;
    #27:
      begin
        FDataLink.Reset;
        SelectAll;
        Key := #0;
      end;
  end;
end;

procedure TCustomDBSpinEdit.Loaded;
begin
  inherited Loaded;
  ResetMaxLength;
  SetDataField(DataField);
  if (csDesigning in ComponentState) then DataChange(Self);
end;

procedure TCustomDBSpinEdit.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TCustomDBSpinEdit.Reset;
begin
  FDataLink.Reset;
  SelectAll;
end;

procedure TCustomDBSpinEdit.ResetMaxLength;
var
  F: TField;
begin
  if (MaxLength > 0) and Assigned(DataSource) and Assigned(DataSource.DataSet) then
  begin
    F := DataSource.DataSet.FindField(DataField);
    if Assigned(F) and (F.DataType in [ftString{$IFDEF POLARIS_D4}, ftWideString{$ENDIF}]) and (F.Size = MaxLength) then
      MaxLength := 0;
  end;
end;

procedure TCustomDBSpinEdit.DownClick(Sender: TObject);
begin
  FDataLink.Edit;
  inherited DownClick(Sender);
end;

procedure TCustomDBSpinEdit.UpClick(Sender: TObject);
begin
  FDataLink.Edit;
  inherited UpClick(Sender);
end;

procedure TCustomDBSpinEdit.WMPaint(var Message: TWMPaint);
const
  AlignStyle : array[Boolean, TAlignment] of DWORD =
   ((WS_EX_LEFT, WS_EX_RIGHT, WS_EX_LEFT),
    (WS_EX_RIGHT, WS_EX_LEFT, WS_EX_LEFT));
var
  Left: Integer;
  Margins: TPoint;
  R: TRect;
  DC: HDC;
  PS: TPaintStruct;
  S: string;
  AAlignment: TAlignment;
{$IFDEF POLARIS_D4}
  ExStyle: DWORD;
{$ENDIF}
begin
  AAlignment := FAlignment;
{$IFDEF POLARIS_D4}
  if UseRightToLeftAlignment then ChangeBiDiModeAlignment(AAlignment);
{$ENDIF}
  if ((AAlignment = taLeftJustify) or FFocused) and
    not (csPaintCopy in ControlState) then
  begin
{$IFDEF POLARIS_D4}
    if SysLocale.MiddleEast and HandleAllocated and (IsRightToLeft) then
    begin { This keeps the right aligned text, right aligned }
      ExStyle := DWORD(GetWindowLong(Handle, GWL_EXSTYLE)) and (not WS_EX_RIGHT) and
        (not WS_EX_RTLREADING) and (not WS_EX_LEFTSCROLLBAR);
      if UseRightToLeftReading then ExStyle := ExStyle or WS_EX_RTLREADING;
      if UseRightToLeftScrollbar then ExStyle := ExStyle or WS_EX_LEFTSCROLLBAR;
      ExStyle := ExStyle or
        AlignStyle[UseRightToLeftAlignment, AAlignment];
      if DWORD(GetWindowLong(Handle, GWL_EXSTYLE)) <> ExStyle then
        SetWindowLong(Handle, GWL_EXSTYLE, ExStyle);
    end;
{$ENDIF}
    inherited;
    Exit;
  end;
{ Since edit controls do not handle justification unless multi-line (and
  then only poorly) we will draw right and center justify manually unless
  the edit has the focus. }
  if FCanvas = nil then
  begin
    FCanvas := TControlCanvas.Create;
    FCanvas.Control := Self;
  end;
  DC := Message.DC;
  if DC = 0 then DC := BeginPaint(Handle, PS);
  FCanvas.Handle := DC;
  try
    FCanvas.Font := Font;
    with FCanvas do
    begin
      R := ClientRect;
      if not (NewStyleControls and Ctl3D) and (BorderStyle = bsSingle) then
      begin
        Brush.Color := clWindowFrame;
        FrameRect(R);
        InflateRect(R, -1, -1);
      end;
      Dec(R.Right, ButtonWidth);
      Brush.Color := Color;
      if not Enabled then
        Font.Color := clGrayText;
      if (csPaintCopy in ControlState) and (FDataLink.Field <> nil) then
      begin
        S := FDataLink.Field.DisplayText;
        case CharCase of
          ecUpperCase: S := AnsiUpperCase(S);
          ecLowerCase: S := AnsiLowerCase(S);
        end;
      end else
        S := EditText;
      if PasswordChar <> #0 then FillChar(S[1], Length(S), PasswordChar);
      Margins := GetTextMargins;
      case AAlignment of
        taLeftJustify: Left := Margins.X;
        taRightJustify: Left := ClientWidth - ButtonWidth - TextWidth(S) - Margins.X - 1;
      else
        Left := (ClientWidth - ButtonWidth - TextWidth(S)) div 2;
      end;
{$IFDEF POLARIS_D4}
      if SysLocale.MiddleEast then UpdateTextFlags;
{$ENDIF}
      TextRect(R, Left, Margins.Y, S);
    end;
  finally
    FCanvas.Handle := 0;
    if Message.DC = 0 then EndPaint(Handle, PS);
  end;
end;

function TCustomDBSpinEdit.GetTextMargins: TPoint;
var
  DC: HDC;
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
begin
  if NewStyleControls then
  begin
    if BorderStyle = bsNone then I := 0 else
      if Ctl3D then I := 1 else I := 2;
    Result.X := SendMessage(Handle, EM_GETMARGINS, 0, 0) and $0000FFFF + I;
    Result.Y := I;
  end else
  begin
    if BorderStyle = bsNone then I := 0 else
    begin
      DC := GetDC(0);
      GetTextMetrics(DC, SysMetrics);
      SaveFont := SelectObject(DC, Font.Handle);
      GetTextMetrics(DC, Metrics);
      SelectObject(DC, SaveFont);
      ReleaseDC(0, DC);
      I := SysMetrics.tmHeight;
      if I > Metrics.tmHeight then I := Metrics.tmHeight;
      I := I div 4;
    end;
    Result.X := I;
    Result.Y := I;
  end;
end;

procedure TCustomDBSpinEdit.SetValue(NewValue: Extended);
var
  FPrecision,
  FDigits: Integer;
  FFormat: TFloatFormat;
  AValueType: TValueType;
begin

  if not Assigned(FDataLink) then Exit;

  AValueType := RealValueType;

  if AValueType = vtFloat then begin
    FFormat := ffGeneral;
    FPrecision := 15;
    FDigits := 0;
    if Assigned(FDataLink) and Assigned(FDataLink.Field) and
       (FDataLink.Field is TFloatField)
    then begin
      FPrecision := TFloatField(FDataLink.Field).Precision;
      if TFloatField(FDataLink.Field).currency then begin
        if not FFocused then FFormat := ffCurrency else FFormat := ffFixed;
        FDigits := CurrencyDecimals;
      end;
    end;
    Text := FloatToStrF(CheckValue(NewValue), FFormat, FPrecision, FDigits)
  end
  else if AValueType = vtHex then
    Text := IntToHex(Round(CheckValue(NewValue)), 1)
  else
    Text := IntToStr(Round(CheckValue(NewValue)));
end;

function TCustomDBSpinEdit.GetValue: Extended;
var
  AValueType: TValueType;
begin

  AValueType := RealValueType;
  try
    if AValueType = vtFloat then begin
      if not TextToFloat(PChar(Text), Result, fvExtended)
      then Result := MinValue;
    end
    else if AValueType = vtHex then Result := StrToIntDef('$' + Text, Round(MinValue))
    else Result := StrToIntDef(Text, Round(MinValue));
  except
    if AValueType = vtFloat then Result := MinValue
    else Result := Round(MinValue);
  end;

end;

procedure TCustomDBSpinEdit.SetValueTypeAuto(Value: Boolean);
begin
  if Value <> FValueTypeAuto then begin
    FValueTypeAuto := Value;
    CheckValueType;
  end;
end;

procedure TCustomDBSpinEdit.SetValueType(NewType: TValueType);
begin
  inherited SetValueType(NewType);
  if RealValueType <> NewType then begin
    if ValueTypeAuto then ValueTypeAuto := False; 
    CheckValueType;
  end;
  if Assigned(FDataLink) then DataChange(Self);
end;

end.

