//==============================================
//      File polarisreg.pas
//
//      Components library for Delphi, BCB++, Kylix.
//      DesignTime (components registration).
//
//      Copyright 1998-2002 Polaris Software
//      http://polesoft.da.ru
//      mailto: PolarisLib@mail.ru
//==============================================
unit PolarisReg;

{$I POLARIS.INC}

interface

uses
  Classes,
  {$IFDEF POLARIS_D6}DesignIntf,DesignEditors{$ELSE}DsgnIntf{$ENDIF};

{$IFNDEF POLARIS_DELPHI5}
type
  TDBStringProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); virtual;
    procedure GetValues(Proc: TGetStrProc); override;
  end;
{$ENDIF}

procedure Register;

implementation

{$R *.dcr}

{$IFDEF LINUX}

uses Classes, rMenuBar, rDBEdit;

procedure Register;
begin
  RegisterComponents('Polaris',
    [TrMenuBar, TrDBEdit]);
end;

{$ELSE}
uses Controls,
     HintProp, {$IFDEF POLARIS_DELPHI5}DBReg,{$ENDIF}
     DBrSpeedButton,DBrBitBtn,DBSpinEdit,DBStaticText,rDBEdit,rLocateDlg,
     rButtons, rSpeedButton, ErrorMes, Launch, rBkgnd, rPgSetup, rPlugins,
     rComboEdit, rPeriodDlg, rPeriodComboEdit, rLabel, rSetupFields, rDLLloader, rGraph, rConnect
     {$IFDEF POLARIS_D4}, rRecGrid, rAction, ActnList, rMenuBar{$ENDIF}
     {$IFDEF BDE}, DBTables, CrossTable, DBMore, rDBConnect{$ENDIF}, rDBComboEdit, rDBGrid,
     ColnEdit;

type
{ TrCrossTable }
  { TSourceFieldProperty }
  TSourceFieldProperty = class(TDBStringProperty)
    function GetDictionaryPropName: string; virtual;
    procedure GetValueList(List: TStrings); override;
  end;

{$IFDEF BDE}
  TDriversProperty = class(TDBStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

  TAliasNameProperty = class(TDBStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

  { TLookupFieldProperty }
  TLookupFieldProperty = class(TDBStringProperty)
    procedure GetValueList(List: TStrings); override;
  end;
{$ENDIF}
{ TDBrSpeedButton }
  { TSBControlFieldProperty }
  TSBControlFieldProperty = class(TDBStringProperty)
    procedure GetValueList(List: TStrings); override;
  end;

{ TDBrBitBtn }
  { TBBControlFieldProperty }
  TBBControlFieldProperty = class(TDBStringProperty)
    procedure GetValueList(List: TStrings); override;
  end;

{ TDBrBitBtn }
  { TBBControlFieldProperty }
  TDBGCheckFieldProperty = class(TDBStringProperty)
    procedure GetValueList(List: TStrings); override;
  end;

{$IFDEF POLARIS_D4}
{ TrDBActions }
  { TrAControlFieldProperty }
  TrAControlFieldProperty = class(TSourceFieldProperty)
    procedure GetValueList(List: TStrings); override;
  end;


{ TrRecordGrid }

  TrRecFieldNameProperty = class(TDBStringProperty)
    procedure GetValueList(List: TStrings); override;
  end;

type
  TRowsEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;
{$ENDIF}

procedure Register;
begin
  RegisterComponents('Polaris DB',
    [TrDBEdit, TDBrSpeedButton, TDBNULLSpeedButton,
     TDBrBitBtn, TDBNULLButton, TDBSwitchButton,
     TDBSpinEdit, TDBStaticText, TrLocateDlg,
     TrSetupFields{$IFDEF BDE}, TrCloneTable, TrCloneQuery, TrCrossTable, TrDBConnect{$ENDIF},
     TrDBComboEdit, TrDBGrid{$IFDEF POLARIS_D4}, TrRecordGrid{$ENDIF}]);
  RegisterComponents('Polaris',
    [TrSpeedButton, TrSeparator, TrLabel, TrBitBtn, TrComboEdit, TErrorHandler, TLauncher,
     TrPeriodComboEdit, TrPageSetupDialog, TrPlugins, {$IFDEF POLARIS_D4}TrMenuBar,{$ENDIF}
     TrBackground, TrDLLloader, TrImage, TrConnect]);
{$IFDEF POLARIS_D4}
  RegisterActions('Polaris DB',[TrDBActions], nil);
{$ENDIF}

  RegisterPropertyEditor(TypeInfo(TCaption), TrLabel, 'Caption', THintProperty);

{$IFDEF BDE}
{ TrDBConnect }
  RegisterPropertyEditor(TypeInfo(String), TrDBConnect, 'Drivers',
                         TDriversProperty);
  RegisterPropertyEditor(TypeInfo(String), TrDBConnect, 'Alias',
                         TAliasNameProperty);
{ TrCrossTable }
  RegisterPropertyEditor(TypeInfo(String), TrCrossTable, 'RowFieldName',
                         TSourceFieldProperty);

  RegisterPropertyEditor(TypeInfo(String), TrCrossTable, 'ColFieldName',
                         TSourceFieldProperty);

  RegisterPropertyEditor(TypeInfo(String), TrCrossTable, 'DataFieldName',
                         TSourceFieldProperty);

  RegisterPropertyEditor(TypeInfo(String), TrCrossTable, 'ColLookupFieldName',
                         TLookupFieldProperty);

  RegisterPropertyEditor(TypeInfo(String), TrCrossTable, 'ColResultFieldName',
                         TLookupFieldProperty);
{$ENDIF}
{ TDBrSpeedButton }
  RegisterPropertyEditor(TypeInfo(String), TDBrSpeedButton, 'ControlDataField',
                         TSBControlFieldProperty);

{ TDBrBitBtn }
  RegisterPropertyEditor(TypeInfo(String), TDBrBitBtn, 'ControlDataField',
                         TBBControlFieldProperty);

{ TrDBGrid }
  RegisterPropertyEditor(TypeInfo(String), TrDBGrid, 'CheckFieldName',
                         TDBGCheckFieldProperty);

{$IFDEF POLARIS_D4}
{ TrActions }
  RegisterPropertyEditor(TypeInfo(String), TrDBActions, 'ControlDataField',
                         TrAControlFieldProperty);

{ TrRecordGrid }
  RegisterPropertyEditor(TypeInfo(String), TRow, 'FieldName',
                         TrRecFieldNameProperty);

  RegisterComponentEditor(TrRecordGrid, TRowsEditor);
{$ENDIF}
end;

{$IFNDEF POLARIS_DELPHI5}
{ TDBStringProperty }
function TDBStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TDBStringProperty.GetValueList(List: TStrings);
begin
end;

procedure TDBStringProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for I := 0 to Values.Count - 1 do Proc(Values[I]);
  finally
    Values.Free;
  end;
end;
{$ENDIF}

{ TSourceFieldProperty }
function TSourceFieldProperty.GetDictionaryPropName: string;
begin
  Result := 'DictionaryItem';
end;

procedure TSourceFieldProperty.GetValueList(List: TStrings);
{$IFDEF BDE}
var
  CrossTable: TrCrossTable;
{$ENDIF}
begin
{$IFDEF BDE}
  CrossTable := TrCrossTable(GetComponent(0));
  if (CrossTable <> nil) and (CrossTable.SourceDataSet <> nil) then
   CrossTable.SourceDataSet.GetFieldNames(List);
{$ENDIF}
end;

{$IFDEF BDE}
procedure TAliasNameProperty.GetValueList(List: TStrings);
begin
  List.Assign((GetComponent(0) as TrDBConnect).Resources);
end;

procedure TDriversProperty.GetValueList(List: TStrings);
begin
  Session.GetDriverNames(List);
end;

procedure TLookupFieldProperty.GetValueList(List: TStrings);
var
  CrossTable: TrCrossTable;
begin
  CrossTable := TrCrossTable(GetComponent(0));
  if (CrossTable <> nil) and (CrossTable.ColLookUpDataSet <> nil) then
    CrossTable.ColLookUpDataSet.GetFieldNames(List);
end;
{$ENDIF}

{ TDBrSpeedButton }
procedure TSBControlFieldProperty.GetValueList(List: TStrings);
var
  SB: TDBrSpeedButton;
begin
  SB := TDBrSpeedButton(GetComponent(0));
  if (SB <> nil) and (SB.ControlDataSource <> nil) and
     (SB.ControlDataSource.DataSet <> nil) then
    SB.ControlDataSource.DataSet.GetFieldNames(List);
end;

{ TDBrBitBtn }
procedure TBBControlFieldProperty.GetValueList(List: TStrings);
var
  BB: TDBrBitBtn;
begin
  BB := TDBrBitBtn(GetComponent(0));
  if (BB <> nil) and (BB.ControlDataSource <> nil) and
     (BB.ControlDataSource.DataSet <> nil) then
    BB.ControlDataSource.DataSet.GetFieldNames(List);
end;

{ TrDBGrid }
procedure TDBGCheckFieldProperty.GetValueList(List: TStrings);
var
  DBG: TrDBGrid;
begin
  DBG := TrDBGrid(GetComponent(0));
  if (DBG <> nil) and (DBG.DataSource <> nil) and
     (DBG.DataSource.DataSet <> nil) then
    DBG.DataSource.DataSet.GetFieldNames(List);
end;

{$IFDEF POLARIS_D4}
{ TrDBActions }
procedure TrAControlFieldProperty.GetValueList(List: TStrings);
var
  rA: TrDBActions;
begin
  rA := TrDBActions(GetComponent(0));
  if (rA <> nil) and (rA.ControlDataSource <> nil) and
     (rA.ControlDataSource.DataSet <> nil) then
    rA.ControlDataSource.DataSet.GetFieldNames(List);
end;

{ TrRecordGrid }

procedure TRowsEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: ShowCollectionEditorClass(Designer, TCollectionEditor, Component,
    TrRecordGrid(Component).Rows, 'Columns');
  end
end;

function TRowsEditor.GetVerb(Index: Integer): String;
begin
  case Index of
    0: Result := 'Rows Editor';
  end;
end;

function TRowsEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

procedure TrRecFieldNameProperty.GetValueList(List: TStrings);
var
  RecordGrid: TrRecordGrid;
begin
  if (GetComponent(0) is TRow) then begin
    RecordGrid := TRow(GetComponent(0)).GetGrid;
    if (RecordGrid <> nil) and (RecordGrid.DataLink <> nil)
        and (RecordGrid.DataLink.Dataset <> nil) then
      RecordGrid.DataLink.Dataset.GetFieldNames(List);
  end;
end;
{$ENDIF}

{$ENDIF}

end.
