//==============================================
//      File rdbconnect.pas
//
//      Components library for Delphi and BCB++.
//      Connect to database with BDE.
//
//      Copyright 1998-2001 Polaris Software
//      http://polesoft.da.ru
//      mailto: PolarisLib@mail.ru
//==============================================
unit rDBConnect;

{$I POLARIS.INC}

interface

uses Windows, Classes,
     SysUtils, DB, DBTables, BDE, rConnect, rDBConst;

type
  TrDBConnect = class(TrCustomConnect)
  private
    FDatabase: TDatabase;
    FDrivers: string;
    procedure SetDrivers(const Value: string);
    function GetAlias: string;
    procedure SetAlias(const Value: string);
    function GetAliasEnabled: Boolean;
    procedure SetAliasEnabled(const Value: Boolean);
    procedure SetDatabase(Value: TDatabase);
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure DoConnect; override;
    function EnableConnect: Boolean; override;
  published
    property Database: TDatabase read FDatabase write SetDatabase;
    property Drivers: string read FDrivers write SetDrivers;
    property Alias: string read GetAlias write SetAlias;
    property AliasEnabled: Boolean read GetAliasEnabled write SetAliasEnabled default True;
    property RegKey;
    property User;
    property UserEnabled;
    property UserRequired;
    property Password;
    property PasswordRequired;
    property ShowLastPassword;
    property AutoLogon;
    property MaxAttempts;
    property Captions;
    property Help;
    property BeforeConnect;
    property AfterConnect;
    property OnError;
  end;

implementation

uses {$IFDEF POLARIS_D6}RxStrUtils{$ELSE}StrUtils{$ENDIF};

{ TrDBConnect }

constructor TrDBConnect.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  AliasEnabled := True;
  Captions.Resource := srConnectDatabase;
  ResourceDataValue := srDatabaseValue;
  with Help do begin
    Dialog   := 10200;
    Resource := 10201;
    User     := 10202;
    Password := 10203;
    OK       := 10204;
    Cancel   := 10205;
  end;
  Session.GetAliasNames(Resources);
end;

procedure TrDBConnect.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDatabase) then
    Database := nil;
end;

procedure TrDBConnect.SetDatabase(Value: TDatabase);
begin
  if FDatabase <> Value then begin
    FDatabase := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TrDBConnect.SetDrivers(const Value: string);
var
  s: string;
  i: Integer;
begin
  if FDrivers <> Value then begin
    FDrivers := Value;
    Session.GetAliasNames(Resources);
    if Value <> '' then begin
      s := ';'+AnsiUpperCase(FDrivers)+';';
      for i := Resources.Count-1 downto 0 do
        if AnsiPos(';'+AnsiUpperCase(Session.GetAliasDriverName(Resources[i]))+';',s) = 0 then
          Resources.Delete(i);
    end;
  end;
end;

function TrDBConnect.GetAlias: string;
begin
  Result := Resource;
end;

procedure TrDBConnect.SetAlias(const Value: string);
var
  paramList: TStringList;
begin
  if Resource <> Value then begin
    Resource := Value;
    if (User = '') and Session.IsAlias(Value) then begin
      paramList := TStringList.Create;
      try
        Session.GetAliasParams(Value,paramList);
        User := AnsiUpperCase(paramList.Values[szUSERNAME]);
      finally
        paramList.Free;
      end;
    end;
  end;
end;

function TrDBConnect.GetAliasEnabled: Boolean;
begin
  Result := ResourceEnabled;
end;

procedure TrDBConnect.SetAliasEnabled(const Value: Boolean);
begin
  ResourceEnabled := Value;
end;

procedure TrDBConnect.DoConnect;
var
  TH: HDBIDB;
  Options: FLDDesc;
begin
  inherited;
  FillChar(Options, SizeOf(Options), #0);
  Options.iOffset := 0;
  Options.iLen := Length(User) + 1;
  StrPCopy(Options.szName, szUSERNAME);
  if FDatabase = nil then SysUtils.Abort;
  Check(DbiOpenDataBase(PChar(StrToOem(Alias)), nil, dbiReadWrite, dbiOpenShared,
                        PChar(StrToOem(Password)), 1, @Options,
                        PChar(StrToOem(User)), TH));
  Check(DbiCloseDatabase(TH));
  with FDatabase do begin
    Close;
    AliasName := Alias;
    LoginPrompt := False;
    Params.Values[szUSERNAME] := User;
    Params.Values[szPASSWORD] := Password;
    Open;
  end;
end;

function TrDBConnect.EnableConnect: Boolean;
begin
  Result := (inherited EnableConnect) or
            (Session.IsAlias(Alias) and (Session.GetAliasDriverName(Alias)=szCFGDBSTANDARD));
end;

end.

