//==============================================
//      File rspeedbutton.pas
//
//      Components library for Delphi and BCB++.
//       SpeedButton  .
//      SpeedButton components and delimiter.
//
//      Copyright 1998-2001 Polaris Software
//      http://members.xoom.com/PolarisSoft
//      mailto: PolarisLib@mail.ru
//==============================================
unit rSpeedButton;

{$I POLARIS.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls,
  Buttons, Menus, rUtils, rVCLUtils, rConst, RxCtrls;

type
  TrSeparatorStyle = (rspSeparator, rspDivider);
  TMenuItemClickOption = (micoNone, micoBefore, micoAfter);

  TrWinControl = class(TWinControl)
  published
    property OnKeyDown;
  end;

  TrSpeedButton = class(TRxSpeedButton)
  private
    { Private declarations }
    FGlyphResKind: TBitmapResKind;
    FGlyphResource : string;
    FMenuItem      : TMenuItem;
    FPopupMenuItem : TMenuItem;
    FMenuShortCut: TShortCut;
    FOldMenuShortCut: TShortCut;
    FOldPopupShortCut: TShortCut;
    FOldMenuEnabled: Boolean;
    FMenuCallOption: TMenuItemClickOption;
    FOldMenuClick  : TNotifyEvent;
    FOldPopupClick : TNotifyEvent;
    FOldPopupEnabled: Boolean;
    FControlFocused:  TWinControl;
    FControlShortCut: TShortCut;
    FOldControlKeyDown: TKeyEvent;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure SetMenuItem(Value: TMenuItem);
    procedure SetPopupMenuItem(Value: TMenuItem);
    procedure SetMenuShortCut(Value: TShortCut);
    procedure SetGlyphResKind(Value: TBitmapResKind);
    procedure SetGlyphResource(Value: string);
    function  IsGlyph: Boolean;
    procedure SetControlFocused(Value: TWinControl);
    procedure SetControlShortCut(Value: TShortCut);
  protected
    { Protected declarations }
    procedure SetMenuEnabled(AMenuItem: TMenuItem);
    procedure Loaded; override;
    procedure InternalClick(Sender: TObject);
    procedure InternalKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  public
    { Public declarations }
    FControlKey: Word;
    FControlShift: TShiftState;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Notification(AComponent: TComponent;
                           Operation: TOperation); override;
  published
    { Published declarations }
    property FlatStandard default True;
    property Align;
    property Flat default True;
    property Transparent default True;
    property MenuItem: TMenuItem read  FMenuItem write SetMenuItem;
    property PopupMenuItem: TMenuItem read  FPopupMenuItem write SetPopupMenuItem;
    property MenuShortCut: TShortCut read FMenuShortCut write SetMenuShortCut
      default 0;
    property Glyph stored IsGlyph;
    property MenuCallOption: TMenuItemClickOption read FMenuCallOption
                                                  write FMenuCallOption
                                                  default micoNone;
    property GlyphResKind:  TBitmapResKind read FGlyphResKind write SetGlyphResKind
                                           default DefBRK;
    property GlyphResource: string read FGlyphResource write SetGlyphResource;
    property ControlFocused: TWinControl read FControlFocused write SetControlFocused;
    property ControlShortCut: TShortCut read FControlShortCut write SetControlShortCut
      default 0;
    property Layout default blGlyphLeft;
  end;

  TrSeparator = class(TBevel)
  private
    FSeparatorStyle: TrSeparatorStyle;
    FUseShape: Boolean;
    procedure SetSeparatorStyle(Value: TrSeparatorStyle);
    procedure SetUseShape(Value: Boolean);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align default alLeft;
    property Width default 8;
    property SeparatorStyle: TrSeparatorStyle read FSeparatorStyle
                                              write SetSeparatorStyle
                                              default rspSeparator;
    property UseShape: Boolean read FUseShape write SetUseShape default False;
  end;

implementation

{ TrSpeedButton }

constructor TrSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Layout          := blGlyphLeft;
  Transparent     := True;
  FFlatStandard   := True;
  FMenuItem       := nil;
  FPopupMenuItem  := nil;
  FOldMenuClick   := nil;
  FOldPopupCLick  := nil;
  FMenuCallOption := micoNone;
  FGlyphResKind := DefBRK;
  FControlShortCut := 0;
  Flat := True;
end;

destructor TrSpeedButton.Destroy;
begin
  MenuItem       := nil;
  PopupMenuItem  := nil;
  ControlFocused := nil;
  inherited Destroy;
end;

function TrSpeedButton.IsGlyph: Boolean;
begin
  Result := (FGlyphResKind = rkNone) or (Trim(FGlyphResource) = '');
end;

procedure TrSpeedButton.SetMenuEnabled(AMenuItem: TMenuItem);
begin
  if Assigned(AMenuItem) then
    AMenuItem.Enabled := Enabled;
end;

procedure TrSpeedButton.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  SetMenuEnabled(FMenuItem);
  SetMenuEnabled(FPopupMenuItem);
end;

procedure TrSpeedButton.Notification(AComponent: TComponent;
                                     Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove)
  then begin
    if (AComponent = FMenuItem) and (FMenuItem <> nil)
    then begin
      MenuItem := nil;
      FOldMenuClick := nil;
    end else
    if (AComponent = FPopupMenuItem) and (FPopupMenuItem <> nil)
    then begin
      PopupMenuItem := nil;
      FOldPopupClick := nil;
    end else
    if AComponent = FControlFocused then begin
      FControlFocused := nil;
      FOldControlKeyDown := nil;
    end
  end
end;

procedure TrSpeedButton.InternalClick(Sender: TObject);
  procedure CallMenuItem;
  begin
    if (Sender=FMenuItem) and Assigned(FOldMenuClick)
    then FOldMenuClick(Self);
    if (Sender=FPopupMenuItem) and Assigned(FOldPopupClick)
    then FOldPopupClick(Self);
  end;
begin
  if Assigned(Sender) and (FMenuCallOption = micoBefore)
  then CallMenuItem;
  Click;
  if Assigned(Sender) and (FMenuCallOption = micoAfter)
  then CallMenuItem;
end;

procedure TrSpeedButton.SetMenuItem(Value: TMenuItem);
begin
  if FMenuItem <> Value then begin
    if (FMenuItem <> nil) then begin
      if Assigned(FOldMenuClick) then begin
        FMenuItem.OnClick := FOldMenuClick;
        FOldMenuClick := nil;
        FMenuItem.Enabled := FOldMenuEnabled;
        FMenuItem.ShortCut := FOldMenuShortCut;
      end;
      FMenuItem.Enabled := FOldMenuEnabled;
    end;
    FMenuItem := Value;
    if Value <> nil then begin
      FOldMenuClick := FMenuItem.OnClick;
      FMenuItem.OnClick := InternalClick;
      FOldMenuEnabled := FMenuItem.Enabled;
      FOldMenuShortCut := FMenuItem.ShortCut;
      if (FMenuItem.ShortCut = 0) and not(csDesigning in ComponentState) then
        FMenuItem.ShortCut := FMenuShortCut;
    end;
    SetMenuEnabled(FMenuItem);
  end;
end;

procedure TrSpeedButton.SetPopupMenuItem(Value: TMenuItem);
begin
  if FPopupMenuItem <> Value then begin
    if (FPopupMenuItem <> nil) then begin
      if Assigned(FOldPopupClick) then begin
        FPopupMenuItem.OnClick := FOldPopupClick;
        FOldPopupClick := nil;
        FPopupMenuItem.Enabled := FOldPopupEnabled;
        FPopupMenuItem.ShortCut := FOldPopupShortCut;
      end;
      FPopupMenuItem.Enabled := FOldPopupEnabled;
    end;
    FPopupMenuItem := Value;
    if Value <> nil then begin
      FOldPopupClick := FPopupMenuItem.OnClick;
      FPopupMenuItem.OnClick := InternalClick;
      FOldPopupEnabled := FPopupMenuItem.Enabled;
      FOldPopupShortCut := FPopupMenuItem.ShortCut;
      if (FPopupMenuItem.ShortCut = 0) and not(csDesigning in ComponentState) then
        FPopupMenuItem.ShortCut := FMenuShortCut;
    end;
    SetMenuEnabled(FPopupMenuItem);
  end;
end;

procedure TrSpeedButton.SetMenuShortCut(Value: TShortCut);
begin
  if FMenuShortCut <> Value then begin
    FMenuShortCut := Value;
    if not(csDesigning in ComponentState) then begin
      if FMenuItem <> nil then
        if FOldMenuShortCut = 0 then
          FMenuItem.ShortCut := FMenuShortCut;
      if FPopupMenuItem <> nil then
        if FOldPopupShortCut = 0 then
          FPopupMenuItem.ShortCut := FMenuShortCut;
    end;
  end;
end;

procedure TrSpeedButton.SetGlyphResource;
begin
  if FGlyphResource <> Value then begin
    FGlyphResource := Value;
    UpdateGlyph(Self, Glyph, FGlyphResKind, FGlyphResource);
  end;
end;

procedure TrSpeedButton.SetGlyphResKind;
begin
  if FGlyphResKind <> Value then begin
    FGlyphResKind := Value;
    UpdateGlyph(Self, Glyph, FGlyphResKind, FGlyphResource);
  end;
end;

procedure TrSpeedButton.Loaded;
begin
  inherited;
  UpdateGlyph(Self, Glyph, FGlyphResKind, FGlyphResource);
end;

procedure TrSpeedButton.InternalKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Assigned(FOldControlKeyDown) then FOldControlKeyDown(Sender,Key,Shift);
  if (Key = FControlKey) and (Shift = FControlShift) and Enabled then Click;
end;

procedure TrSpeedButton.SetControlFocused(Value: TWinControl);
begin
  if FControlFocused <> Value then begin
    if Assigned(FControlFocused) and Assigned(FOldControlKeyDown) then begin
      TrWinControl(FControlFocused).OnKeyDown := FOldControlKeyDown;
      FOldControlKeyDown := nil;
    end;
    FControlFocused := Value;
    if Assigned(FControlFocused) and (FControlShortCut<>0) then begin
      FOldControlKeyDown := TrWinControl(FControlFocused).OnKeyDown;
      TrWinControl(FControlFocused).OnKeyDown := InternalKeyDown;
    end;
  end;
end;

procedure TrSpeedButton.SetControlShortCut(Value: TShortCut);
begin
  if FControlShortCut <> Value then begin
    FControlShortCut := Value;
    ShortCutToKey(Value,FControlKey,FControlShift);
  end;
end;

{ TrSeparator }

constructor TrSeparator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alLeft;
  Width := 8;
  FSeparatorStyle := rspSeparator;
  FUseShape := False;
end;

procedure TrSeparator.SetSeparatorStyle(Value: TrSeparatorStyle);
begin
  if (Align in [alNone, alClient]) and (Value = rspDivider)
  then raise Exception.Create(srSepError);
  FSeparatorStyle := Value;
  Invalidate;
end;

procedure TrSeparator.SetUseShape(Value: Boolean);
begin
  if Value <> FUseShape then begin
    FUseShape := Value;
    Invalidate;
  end;
end;

procedure TrSeparator.Paint;
var
  R: TRect;
begin
  if (FSeparatorStyle = rspDivider) and not (Align in [alNone, alClient]) then
    with Canvas do
      if Align in [alLeft, alRight] then begin
        R := Rect(Width div 2 - 1, 1, Width, Height - 1);
        DrawEdge(Handle, R, EDGE_ETCHED, BF_Left);
      end else begin
        R := Rect( 1, Height div 2 - 1, Width - 1, Height);
        DrawEdge(Handle, R, EDGE_ETCHED, BF_Top);
      end;
  if FUseShape then inherited
  else
    if csDesigning in ComponentState then
      with Canvas do begin
        Pen.Style := psDot;
        Pen.Color := clBtnShadow;
        Brush.Style := bsClear;
        Rectangle(0, 0, ClientWidth, ClientHeight);
        Pen.Style := psSolid;
        Brush.Style := bsSolid;
      end;
end;

end.
