//==============================================
//      File rvclutils.pas
//
//      Components library for Delphi and BCB++.
//      ,    (VCL).
//      Procedures, functions and types.
//
//      Copyright 1998-2002 Polaris Software
//      http://polesoft.da.ru
//      mailto: PolarisLib@mail.ru
//==============================================
unit rVCLUtils;

{$I POLARIS.INC}

interface

uses Classes, Windows, SysUtils, Graphics, Consts, Registry, Controls, Forms,
     {$IFDEF POLARIS_D6}RxStrUtils, Variants, RTLConsts{$ELSE}StrUtils{$ENDIF},
     {$IFDEF GIF}RxGIF,{$ENDIF}{$IFDEF JPEG}JPEG,{$ENDIF} Menus, rUtils, rConst;

//========    windows

//     
// Bring to front form
procedure RestoreWindow(Control: TWinControl);

//  
// Redraw window
procedure RefreshWindow(Control: TWinControl; Rect: Pointer);

//    
// Change control's Enabled property
procedure ActiveControls(Owner: TWinControl; Active: Boolean);

//      ImageList'a
// Draw image in canvas from ImageList
procedure DrawImage(Canvas: TCanvas; Rect: TRect; Images: TImageList; Index: Integer);

//========   forms

//     
// Create and terminate modal window
function ExecuteForm(Form: TForm): TModalResult;

//  MDI    
// Check of existing MDI form
function CheckMDIForm(Parent: TForm;ChildName: string): Boolean;

//    
// Standard MessageBox dialog
function rMsgBox(Text: string; Flags: Longint): Integer;

//    
// Confirm cancel changes
function ConfirmCancel(Caption: string): Boolean;

//   Hint
// Forced hint showing
procedure AppHint(StrHint: string);

//========   resources

//     RHandles
// Search resource in RHandles array
function FindResourceInModules(var Module: THandle; const ResName, ResType: PChar): HRSRC;

//   
// Correct name of resource
procedure CorrectResName(var ResName: string);

//    
// Load image from resource
function RLoadPicture(RHandle: THandle; ResName: string; ResKind: TBitmapResKind): THANDLE;

//        
// Load image from resource with converting in bitmap
procedure LoadBitmapFromResource(Bitmap: TBitmap; RHandle: THandle;
                                 ResName: string; ResKind: TBitmapResKind);

//  Bitmap'  ImageList  
// Load bitmap to ImageList from resource
function LoadToImage(Handle: THandle; ResName: PChar; ResKind: TBitmapResKind; Image: TImageList): Boolean;

// For Polaris button glyphs!
procedure UpdateGlyph(Comp: TComponent; Glyph: TBitmap; GlyphResKind: TBitmapResKind; GlyphResource: string);

//========   registry

//  
// Read value
function RegReadValue(Reg: TRegistry; Name: string; Default: string {$IFDEF POLARIS_D4}= ''{$ENDIF}): string;
{$IFDEF POLARIS_D4}overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: Integer = 0): Integer; overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: Extended = 0): Extended; overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: Boolean = False): Boolean; overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: TDateTime = 0): TDateTime; overload;
{$ENDIF}

//   Source  RootKey  RegKey
// Separate string Source to RootKey and RegKey
function SeparateRegKey(Source: String; var TargetRoot: HKEY; var TargetKey: String): Boolean;

//========    other

//   
// Copy menu items
procedure CopyMenu(Source, Dest: TMenuItem);

//  TFont    
// Convert TFont into delimited string
function FontToStr(Font: TFont; Delim: string): string;

//    TFont
// Convert delimited string into TFont
procedure StrToFont(S: string; Font: TFont; Delim: Char);

implementation

procedure RestoreWindow(Control: TWinControl);
var
  Action: Integer;
begin
  if Assigned(Control) then begin
    Action := SW_SHOW;
    if IsZoomed(Control.Handle) then Action := SW_MAXIMIZE
    else if IsIconic(Control.Handle) then Action := SW_RESTORE;
    ShowWindowAsync(Control.Handle,Action);
    BringWindowToTop(Control.Handle);
  end;
end;

procedure RefreshWindow(Control: TWinControl; Rect: Pointer);
begin
  if Assigned(Control)
  then RedrawWindow(Control.Handle, Rect, 0,
          RDW_ERASENOW or RDW_UPDATENOW or RDW_FRAME or RDW_ALLCHILDREN or RDW_INVALIDATE);
end;

procedure ActiveControls(Owner: TWinControl; Active: Boolean);
var i: Integer;
begin
  with Owner do
    for i:=0 to ControlCount-1 do
      if (Controls[i] is TWinControl) or
         (Controls[i] is TGraphicControl) then begin
        Controls[i].Enabled := Active;
      end;
end;

procedure DrawImage(Canvas: TCanvas; Rect: TRect; Images: TImageList; Index: Integer);
begin
  Canvas.FillRect(Rect);
  if Index < 0
  then Exit;
  if ((Rect.Right-Rect.Left+2)>=Images.Width)
  then
    Images.Draw(Canvas,(Rect.Right+Rect.Left-Images.Width-2) shr 1,
                       (Rect.Top+Rect.Bottom-Images.Height) shr 1, Index);
end;

function ExecuteForm(Form: TForm): TModalResult;
begin
  try
    Result := Form.ShowModal;
  finally
    Form.Free;
  end;
end;

function CheckMDIForm(Parent: TForm;ChildName: string): Boolean;
var i: Integer;
begin
  Result := False;
  for i:=0 to Parent.MDIChildCount-1 do
    if UpperCase(Parent.MDIChildren[i].Name) = UpperCase(ChildName) then begin
      Result := True;
      exit;
    end;
end;

function rMsgBox(Text: string; Flags: Longint): Integer;
begin
  Result := Application.MessageBox(PChar(Text),PChar(Application.Title),Flags);
end;

function ConfirmCancel(Caption: string): Boolean;
begin
  Result := Application.MessageBox(PChar(ConvertCodes(srFormCancel)),
                PChar(Caption), MB_ICONQUESTION+MB_OKCANCEL) = IDOK
end;

procedure AppHint(StrHint: string);
begin
  with Application do
    if Assigned(OnHint) then begin
      Hint := StrHint;
      OnHint(Application);
    end;
end;

function FindResourceInModules(var Module: THandle; const ResName, ResType: PChar): HRSRC;
var
  i: Integer;
begin
  Result := FindResource(Module, ResName, ResType);
  if Result > 0 then exit;
  for i:=0 to RHandles.Count-1 do begin
    Result := FindResource(THandle(RHandles.Items[i]), ResName, ResType);
    if Result > 0 then begin
      Module := THandle(RHandles.Items[i]);
      exit;
    end;
  end;
end;

procedure CorrectResName(var ResName: string);
var
  ResID: Integer;
begin
  ResName := Trim(ResName);
  ResID := StrToIntDef(ResName,0);
  if ResID > 0 then ResName := '#'+ResName;
end;

function RLoadPicture(RHandle: THandle; ResName: string; ResKind: TBitmapResKind): THANDLE;
begin
  CorrectResName(ResName);
  if FindResourceInModules(RHandle,PChar(ResName),ResKindNames[ResKind]) = 0 then
    raise EResNotFound.CreateFmt(SResNotFound,[ResName]);
  if ResName[1] = '#' then
    ResName := String(MakeIntResource(Copy(ResName,2,Length(ResName))));
  case ResKind of
    rkBitmap: Result := LoadBitmap(RHandle,PChar(ResName));
    rkIcon:   Result := LoadIcon(RHandle,PChar(ResName));
    rkCursor: Result := LoadCursor(RHandle,PChar(ResName));
  else
    Result := 0;
  end;
end;

procedure LoadBitmapFromResource(Bitmap: TBitmap; RHandle: THandle;
                                 ResName: string; ResKind: TBitmapResKind);
type
  TCG = class of TGraphic;
var
  CG: TCG;
  Image: TGraphic;
  Stream: TResourceStream;
begin
  CorrectResName(ResName);
  if FindResourceInModules(RHandle,PChar(ResName),ResKindNames[ResKind]) = 0 then
    raise EResNotFound.CreateFmt(SResNotFound,[ResName]);
  case ResKind of
    rkBitmap: Bitmap.LoadFromResourceName(RHandle,ResName);
    {rkIcon,}rkEMF,rkWMF{$IFDEF GIF},rkGIF{$ENDIF}{$IFDEF JPEG},rkJPEG{$ENDIF}:
      begin
        Stream := TResourceStream.Create(RHandle,ResName,ResKindNames[ResKind]);
        try
          case ResKind of
//            rkIcon:      CG := TIcon;
            rkEMF,rkWMF: CG := TMetafile;
            {$IFDEF GIF}rkGIF: CG := TGIFImage;{$ENDIF}
            {$IFDEF JPEG}rkJPEG: CG := TJPEGImage;{$ENDIF}
          else
            CG := nil;
          end;
          Image := CG.Create;
          try
            Image.LoadFromStream(Stream);
            if ResKind = rkEMF then TMetafile(Image).Enhanced := True;
            with Image do begin
              Bitmap.Width := Width;
              Bitmap.Height := Height;
            end;
            Bitmap.Canvas.Draw(0,0,Image);
          finally
            Image.Free;
          end;
        finally
          Stream.Free;
        end;
      end;
   rkIcon:
      begin
        Image := TIcon.Create;
        with Image as TIcon do
          try
            Handle := RLoadPicture(RHandle,ResName,rkIcon);
            if not Empty then
              with Bitmap do begin
                Width  := 32;
                Height := 32;
                Canvas.Brush.Color := clFuchsia;
                Canvas.FillRect(Canvas.ClipRect);
                Canvas.Draw(0,0,Image);
              end;
          finally
            Free;
          end;
      end;
  end;
end;

function LoadToImage(Handle: THandle; ResName: PChar; ResKind: TBitmapResKind; Image: TImageList): Boolean;
var
  Bmp: TBitmap;
begin
  Result := False;
  if Assigned(Image)
  then begin
    Bmp := TBitmap.Create;
    try
      LoadBitmapFromResource(Bmp,Handle,ResName,ResKind);
      Image.AddMasked(Bmp,Bmp.TransparentColor);
      Result := True;
    finally
      Bmp.Free;
    end
  end
end;

procedure UpdateGlyph(Comp: TComponent; Glyph: TBitmap; GlyphResKind: TBitmapResKind; GlyphResource: string);
begin
  if (GlyphResKind = rkNone) or (Trim(GlyphResource) = '') or (csReading in Comp.ComponentState) then exit;
  try
    LoadBitmapFromResource(Glyph, hInstance, GlyphResource, GlyphResKind);
  except
    on EResNotFound do if not(csDesigning in Comp.ComponentState) then raise;
    else raise;
  end;
end;

function RegReadValue(Reg: TRegistry; Name: string; Default: string): string;
{$IFDEF POLARIS_D4}overload;{$ENDIF}
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadString(Name)
  else Result := Default;
end;
{$IFDEF POLARIS_D4}
function RegReadValue(Reg: TRegistry; Name: string; Default: Integer): Integer; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadInteger(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TRegistry; Name: string; Default: Extended): Extended; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadFloat(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TRegistry; Name: string; Default: Boolean): Boolean; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadBool(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TRegistry; Name: string; Default: TDateTime): TDateTime; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadDateTime(Name)
  else Result := Default;
end;
{$ENDIF}

function SeparateRegKey(Source: String; var TargetRoot: HKEY; var TargetKey: String): Boolean;
var
  i: Integer;
  s: String;
begin
  TargetRoot := HKEY_CURRENT_USER;
  TargetKey := Source;
  Result := False;
  if (Length(Source) < 2) or (Source[1] = '\') then Exit;
  i := pos('\', Source);
  if i = 0 then Exit;
  s := ExtractWord(1, Source, ['\']);
  if (s = 'HKEY_CLASSES_ROOT') or     (s = 'HKCR') then TargetRoot := HKEY_CLASSES_ROOT
  else
  if (s = 'HKEY_CURRENT_USER') or     (s = 'HKCU') then TargetRoot := HKEY_CURRENT_USER
  else
  if (s = 'HKEY_LOCAL_MACHINE') or    (s = 'HKLM') then TargetRoot := HKEY_LOCAL_MACHINE
  else
  if (s = 'HKEY_USERS') or            (s = 'HKUS') then TargetRoot := HKEY_USERS
  else
  if (s = 'HKEY_PERFORMANCE_DATA') or (s = 'HKPD') then TargetRoot := HKEY_PERFORMANCE_DATA
  else
  if (s = 'HKEY_CURRENT_CONFIG') or   (s = 'HKCC') then TargetRoot := HKEY_CURRENT_CONFIG
  else
  if (s = 'HKEY_DYN_DATA') or         (s = 'HKDD') then TargetRoot := HKEY_DYN_DATA
  else Exit;
  TargetKey := Copy(Source, i, Length(Source)-i+1);
  Result := True;
end;

procedure CopyMenu(Source, Dest: TMenuItem);
var
  i: Integer;
  CurrItem, NewItem: TMenuItem;
begin
{$IFDEF POLARIS_D5}
  Dest.Clear;
{$ELSE}
  for i := Dest.Count - 1 downto 0 do begin
    CurrItem := Dest.Items[i];
    Dest.Delete(i);
    CurrItem.Free;
  end;
{$ENDIF}
  for i := 0 to Source.Count - 1 do begin
    CurrItem := Source.Items[i];
    NewItem := TMenuItem.Create(Dest);
    CopyObject(CurrItem, NewItem);
    Dest.Add(NewItem);
  end;
end;

function FontToStr(Font: TFont; Delim: string): string;
begin
  Result := '';
  with Font do begin
    if fsBold in Style then Result := Result + 'B';
    if fsItalic in Style then Result := Result + 'I';
    if fsUnderline in Style then Result := Result + 'U';
    if fsStrikeOut in Style then Result := Result + 'S';
    Result := Result + Delim + IntToStr(Size) +
              Delim + ColorToString(Color) +
              Delim + Name +
              Delim + IntToStr(Charset);
  end;
end;

procedure StrToFont(S: string; Font: TFont; Delim: Char);
var
  i: Integer;
  sb: string;
begin
  S := '';
  with Font do
    for i := 1 to 5 do begin
      sb := ExtractDelimited(i, S, [Delim]);
      case i of
        1: begin
             Style := [];
             if Pos('B', sb) <> 0 then Style := Style + [fsBold];
             if Pos('I', sb) <> 0 then Style := Style + [fsItalic];
             if Pos('U', sb) <> 0 then Style := Style + [fsUnderline];
             if Pos('S', sb) <> 0 then Style := Style + [fsStrikeOut];
           end;
        2: Size := StrToIntDef(sb, 8);
        3: Color := StringToColor(sb);
        4: Name := sb;
        5: Charset := StrToIntDef(sb, 1);
      end;
    end;
end;

initialization
{$IFDEF POLARIS_D6}
  RHandles.Add(Pointer(GetModuleHandle('vcl60.bpl')));
  RHandles.Add(Pointer(GetModuleHandle('rxctl6.bpl')));
{$ELSE}
{$IFDEF POLARIS_D5}
  RHandles.Add(Pointer(GetModuleHandle('vcl50.bpl')));
  RHandles.Add(Pointer(GetModuleHandle('rxctl5.bpl')));
{$ELSE}
{$IFDEF POLARIS_D4}
  RHandles.Add(Pointer(GetModuleHandle('vcl40.bpl')));
  RHandles.Add(Pointer(GetModuleHandle('rxctl4.bpl')));
{$ELSE}
  RHandles.Add(Pointer(GetModuleHandle('vcl30.dpl')));
  RHandles.Add(Pointer(GetModuleHandle('rxctl.dpl')));
{$ENDIF}
{$ENDIF}
{$ENDIF}
end.
